#!/usr/bin/env perl
# pbo-gen-makefile.pl - generate a makefile for specified pbo-c sources

# call as: pbo-gen-makefile.pl <pbo-c source file 1> <pbo-c source file 2> ...
# input: <pbo-source file>
# output: Makefile.pbo-example

# (C) Quinn Hsu, University of British Columbia 2012-02

# version 0.1
#	- initial port from the weaver

my($ext) = ".pbo-c";

# save the names of the input filenames
my(@inputFiles) = ();

my($makefilename);

# while command line arguments have the right 
# file extension
for my $arg (@ARGV) {
	if ($arg =~ /^(.+$ext)$/) {
		push(@inputFiles, $1);
		unless (-e $1) {
			printf "ERROR: Specified PbO source $1 does not exist.  Please check input file list.\n";
			exit(-1);
		}
	}
	
	if ($arg =~ /^--rename$/) { $makefilename = "Makefile"; } 
	else { $makefilename = "Makefile.pbo-example"; }
}

if (@inputFiles<1) { print "ERROR: Could not detect any $ext files.  Please check input file list\n"; exit(-1); }

my $filenames = join (" ", @inputFiles);

open (MAKEFILE, ">$makefilename");
print MAKEFILE <<"END";

.SUFFIXES: .pbo-c .c .o

# do not delete intermediate .c generated by weaver
.SECONDARY:
SOURCES	= $filenames
END

print MAKEFILE <<'END';

CSOURCES = $(SOURCES:.pbo-c=.c)
HIDDENS = $(addprefix ., $(CSOURCES))
CFILES	= $(HIDDENS) pbo_support.c
OFILES	= $(CFILES:.c=.o)

MAIN	= main
CC      = gcc

all: ${MAIN}.exe

opt.o: opt.c opt.h
	${CC} -c opt.c

$(HIDDENS) : pbo-weave-c.pl

$(CFILES): $(CSOURCES)

.pbo-c.c:
	perl pbo-weave-c.pl $(SOURCES)

.c.o:
	${CC} -o $@ -c $<

${MAIN}.exe: $(OFILES) opt.o
	${CC} -o ${MAIN}.exe  $(OFILES) opt.o

clean:
	rm -f .*.o $(CFILES) pbo_support.h ${MAIN}.exe

END

close(MAKEFILE);